/* Embedded version of hitlib */
#include "hitlib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

static int T = -1;
static long long N_local = -1;
static const long long embedded_data[10] = {
  18393220076LL,
  2488283021LL,
  23249390331LL,
  17408627799LL,
  14802861370LL,
  12179525861LL,
  6124404437LL,
  3518622047LL,
  10978803143LL,
  13508197171LL
};

void Initialize(int Targ) {
  if(Targ < 0 || Targ > 15) {
    fprintf(stderr, "ERR 1 Invalid T.\n");
    exit(1);
  }
  if(T != -1) {
    fprintf(stderr, "ERR 3 Initialization called twice.\n");
    exit(3);
  }
  T = Targ;
  if(T == 0) {
    FILE *hit_in = fopen("hit.in", "rb");
    if(!hit_in) {
      int last_errno = errno;
      fprintf(stderr, "ERR 5 File error.\n");
      fprintf(stderr, "Detail: Can't open hit.in: %s.\n", strerror(last_errno));
      exit(5);
    }
    if(fscanf(hit_in, "%lld\n", &N_local) < 1) {
      if(ferror(hit_in)) {
        int last_errno = errno;
        fprintf(stderr, "ERR 5 File error.\n");
        fprintf(stderr, "Detail: Can't read from hit.in: %s.\n",
            strerror(last_errno));
        fclose(hit_in);
        exit(5);
      } else {
        fprintf(stderr, "ERR 5 File error.\n");
        fprintf(stderr, "Detail: Can't read from hit.in.\n");
        fclose(hit_in);
        exit(5);
      }
    }
    if(fclose(hit_in)) {
      int last_errno = errno;
      fprintf(stderr, "ERR 5 File error.\n");
      fprintf(stderr, "Detail: Can't close hit.in: %s.\n", strerror(last_errno));
      exit(5);
    }
    return;
  }
}
int Hit(long long M) {
  if(M < 0 || M >= 10000000000LL) {
    fprintf(stderr, "ERR 2 Invalid M.\n");
    exit(2);
  }
  if(T == -1) {
    fprintf(stderr, "ERR 4 Hit called before Initialization.\n");
    exit(4);
  }
  if(T == 0) {
    int cnt = 0;
    long long N = N_local;
    int i;
    for(i = 0; i < 10; ++i) {
      if(N%10 == M%10) {
        ++cnt;
      }
      N /= 10;
      M /= 10;
    }
    return cnt;
  } else {
    int cnt = 0;
    long long N = embedded_data[T-1];
    int i;
    for(i = 0; i < 10; ++i) {
      if(N%11 == M%10) {
        ++cnt;
      }
      N /= 11;
      M /= 10;
    }
    return cnt;
  }
}
void Finalize(void) {
}
